searchNodes=[{"ref":"XDR.html","title":"XDR","type":"module","doc":"Basic XDR usage"},{"ref":"XDR.html#build_type/2","title":"XDR.build_type/2","type":"function","doc":"Build a concrete XDR type by providing the type module and any configuration options for that type. Examples Some types don&#39;t take any configuration. These include XDR.Type.Bool XDR.Type.Double XDR.Type.Float XDR.Type.HyperInt XDR.Type.Int XDR.Type.UnsignedHyperInt XDR.Type.UnsignedInt XDR.Type.Void Examples: iex&gt; XDR.build_type(XDR.Type.Int) %XDR.Type.Int{type_name: &quot;Int&quot;, value: nil} iex&gt; XDR.build_type(XDR.Type.Float) %XDR.Type.Float{type_name: &quot;Float&quot;, value: nil} XDR.Type.VariableOpaque and XDR.Type.String have an optional max_length option, with a default max defined in XDR.Size. iex&gt; XDR.build_type(XDR.Type.VariableOpaque) %XDR.Type.VariableOpaque{type_name: &quot;VariableOpaque&quot;, max_length: XDR.Size.max(), value: nil} iex&gt; XDR.build_type(XDR.Type.VariableOpaque, 100) %XDR.Type.VariableOpaque{type_name: &quot;VariableOpaque&quot;, max_length: 100, value: nil} XDR.Type.VariableArray has a required type and an optional max_length. iex&gt; XDR.build_type(XDR.Type.VariableArray, type: XDR.build_type(XDR.Type.Int)) %XDR.Type.VariableArray{type_name: &quot;VariableArray&quot;, data_type: %XDR.Type.Int{}, max_length: XDR.Size.max(), values: []} XDR.Type.Array and XDR.Type.Opaque are fixed-length, so the length is required when building the type: iex&gt; XDR.build_type(XDR.Type.Array, type: XDR.build_type(XDR.Type.Bool), length: 4) %XDR.Type.Array{type_name: &quot;Array&quot;, data_type: %XDR.Type.Bool{}, length: 4, values: []} iex&gt; XDR.build_type(XDR.Type.Opaque, 4) %XDR.Type.Opaque{type_name: &quot;Opaque&quot;, length: 4, value: nil} iex&gt; XDR.build_type(XDR.Type.Opaque) ** (XDR.Error) A valid size must be provided XDR.Type.Enum is an enumeration with atom keys and signed int values, provided as a keyword list iex&gt; _enum_type = XDR.build_type(XDR.Type.Enum, metric: 0, imperial: 1, other: 2) %XDR.Type.Enum{type_name: &quot;Enum&quot;, options: [metric: 0, imperial: 1, other: 2], value: nil} Creating a concrete XDR.Type.Optional type requires that a fully built base type be provided: iex&gt; XDR.build_type(XDR.Type.Optional, XDR.build_type(XDR.Type.Float)) %XDR.Type.Optional{type_name: &quot;Optional&quot;, data_type: %XDR.Type.Float{}, value: nil} XDR.Type.Struct allows the nesting of data into trees, by associating a key with an XDR data type to be stored under that key. iex&gt; _user_type = XDR.build_type( ...&gt; XDR.Type.Struct, ...&gt; first_name: XDR.build_type(XDR.Type.VariableOpaque), ...&gt; last_name: XDR.build_type(XDR.Type.VariableOpaque), ...&gt; email: XDR.build_type(XDR.Type.VariableOpaque) ...&gt; ) %XDR.Type.Struct{type_name: &quot;Struct&quot;, fields: [ first_name: %XDR.Type.VariableOpaque{}, last_name: %XDR.Type.VariableOpaque{}, email: %XDR.Type.VariableOpaque{} ]} XDR.Type.Union is a discriminated union, with its main data type being determined the the value of its switch. The switch can optionally be given a name, and must be of type XDR.Type.Bool, XDR.Type.Enum, XDR.Type.Int, or XDR.Type.UnsignedInt. Each entry in the switches list maps the switch values to one of the arms, or in some cases directly to XDR.Type.Void if no value is needed. The arms themselves store the XDR type the union will take in that case. iex&gt; XDR.build_type( ...&gt; XDR.Type.Union, ...&gt; switch_name: &quot;UserType&quot;, ...&gt; switch_type: XDR.build_type(XDR.Type.Enum, business: 0, consumer: 1, none: 2), ...&gt; switches: [ ...&gt; business: :business_account, ...&gt; consumer: :consumer_account, ...&gt; none: XDR.Type.Void ...&gt; ], ...&gt; arms: [ ...&gt; business_account: XDR.build_type(XDR.Type.Opaque, 16), ...&gt; consumer_account: XDR.build_type(XDR.Type.Int) ...&gt; ] ...&gt; ) %XDR.Type.Union{ switch_name: &quot;UserType&quot;, switch: %XDR.Type.Enum{options: [business: 0, consumer: 1, none: 2]}, switches: [ business: :business_account, consumer: :consumer_account, none: XDR.Type.Void ], arms: [ business_account: %XDR.Type.Opaque{length: 16}, consumer_account: %XDR.Type.Int{} ], type_name: &quot;Union&quot; } Building data types on the fly isn&#39;t suitable for a complex problem domain, so XDR.Base is provided to allow an application to pre-define named XDR types for use throughout the application. See the readme for more info."},{"ref":"XDR.html#build_value/2","title":"XDR.build_value/2","type":"function","doc":"To build a concrete value, supply the fully-built type and a value appropriate to that type&#39;s definition. For simple types, just supply the raw value: iex&gt; int_type = XDR.build_type(XDR.Type.Int) ...&gt; {:ok, int_val} = XDR.build_value(int_type, 123) ...&gt; int_val.value 123 iex&gt; us_zip_type = XDR.build_type(XDR.Type.Opaque, 5) ...&gt; {:ok, zip_val} = XDR.build_value(us_zip_type, &quot;70119&quot;) ...&gt; zip_val.value &quot;70119&quot; iex&gt; enum_type = XDR.build_type(XDR.Type.Enum, metric: 0, imperial: 1, other: 2) ...&gt; {:ok, enum_val} = XDR.build_value(enum_type, :metric) ...&gt; enum_val.value :metric iex&gt; bool_type = XDR.build_type(XDR.Type.Bool) ...&gt; {:ok, bool_value} = XDR.build_value(bool_type, true) ...&gt; bool_value.value true Arrays work similarly. Just supply a list of appropriate values: iex&gt; scores_type = XDR.build_type(XDR.Type.VariableArray, type: XDR.build_type(XDR.Type.Int)) ...&gt; {:ok, scores} = XDR.build_value(scores_type, [1, 2, 3, 4, 5, 6]) ...&gt; Enum.map(scores.values, &amp; &amp;1.value) [1, 2, 3, 4, 5, 6] When building a struct&#39;s value, we supply the raw values of the inner types: iex&gt; user_type = XDR.build_type(XDR.Type.Struct, ...&gt; name: XDR.build_type(XDR.Type.VariableOpaque), ...&gt; email: XDR.build_type(XDR.Type.VariableOpaque) ...&gt; ) ...&gt; {:ok, value} = XDR.build_value(user_type, name: &quot;Marvin&quot;, email: &quot;marvin@megadodo.co&quot;) ...&gt; value.fields[:name].value &quot;Marvin&quot; An optional type can be specified in a few different ways for convenience: iex&gt; int_type = XDR.build_type(XDR.Type.Int) ...&gt; optional_int = XDR.build_type(XDR.Type.Optional, int_type) ...&gt; {:ok, no_val_1} = XDR.build_value(optional_int, nil) ...&gt; {:ok, no_val_2} = XDR.build_value(optional_int, false) ...&gt; {:ok, no_val_3} = XDR.build_value(optional_int, {false, &quot;ignored&quot;}) ...&gt; {:ok, with_val_1} = XDR.build_value(optional_int, {true, 123}) ...&gt; {:ok, with_val_2} = XDR.build_value(optional_int, 123) ...&gt; [no_val_1.value, no_val_2.value, no_val_3.value, with_val_1.value.value, with_val_2.value.value] [%XDR.Type.Void{}, %XDR.Type.Void{}, %XDR.Type.Void{}, 123, 123] To build a value for XDR.Type.Union, supply a tuple including the switch value (an int or atom), followed by the value of the corresponding inner type. If the inner type is XDR.Type.Void, then the switch value alone is enough. iex&gt; account_id_type = XDR.build_type( ...&gt; XDR.Type.Union, ...&gt; switch_name: &quot;UserType&quot;, ...&gt; switch_type: XDR.build_type(XDR.Type.Enum, business: 0, consumer: 1, none: 2), ...&gt; switches: [ ...&gt; business: :business_account, ...&gt; consumer: :consumer_account, ...&gt; none: XDR.Type.Void ...&gt; ], ...&gt; arms: [ ...&gt; business_account: XDR.build_type(XDR.Type.Opaque, 16), ...&gt; consumer_account: XDR.build_type(XDR.Type.Int) ...&gt; ] ...&gt; ) ...&gt; {:ok, business_id} = XDR.build_value(account_id_type, {:business, &quot;0123456789abcdef&quot;}) ...&gt; {:ok, consumer_id} = XDR.build_value(account_id_type, {:consumer, 23456}) ...&gt; {:ok, no_id} = XDR.build_value(account_id_type, {:none, nil}) ...&gt; {:ok, no_id_2} = XDR.build_value(account_id_type, :none) ...&gt; [business_id.value.value, consumer_id.value.value, no_id.value, no_id_2.value] [&quot;0123456789abcdef&quot;, 23456, %XDR.Type.Void{}, %XDR.Type.Void{}] NOTE: in all of these examples, the underlying values are accessed directly, which requires some knowledge of the underlying XDR.Type structs. In practice, it&#39;s better to use XDR.extract_value/1 rather than reaching into these structs."},{"ref":"XDR.html#build_value!/2","title":"XDR.build_value!/2","type":"function","doc":"Just like XDR.build_value/2, but returns raw values on success instead of tuples, and raises on failure."},{"ref":"XDR.html#decode/2","title":"XDR.decode/2","type":"function","doc":"Decode a binary representation into an XDR type with value. Since the binary representation does not contain type info itself, it must be supplied as the first parameter. iex&gt; encoding = &lt;&lt;0, 0, 0, 6&gt;&gt; &lt;&gt; &quot;abcdef&quot; &lt;&gt; &lt;&lt;0, 0&gt;&gt; ...&gt; {:ok, type_with_value} = XDR.decode(XDR.build_type(XDR.Type.VariableOpaque), encoding) ...&gt; {type_with_value.length, type_with_value.value} {6, &quot;abcdef&quot;} iex&gt; encoding = &quot;abcdef&quot; &lt;&gt; &lt;&lt;0, 0&gt;&gt; ...&gt; {:ok, type_with_value} = XDR.decode(XDR.build_type(XDR.Type.Opaque, 6), encoding) ...&gt; {type_with_value.length, type_with_value.value} {6, &quot;abcdef&quot;} As with XDR.build_value/2 above, we&#39;re accessing the values directly inside the type structs. A more practical way to access inner values is to use XDR.extract_value/1."},{"ref":"XDR.html#decode!/2","title":"XDR.decode!/2","type":"function","doc":"Just like XDR.decode/2, but returns raw values on success instead of tuples, and raises on failure."},{"ref":"XDR.html#encode/1","title":"XDR.encode/1","type":"function","doc":"Encode an XDR value (created with e.g. XDR.build_value/2) into its binary representation. iex&gt; {:ok, value} = XDR.build_value(XDR.build_type(XDR.Type.Opaque, 6), &quot;abcdef&quot;) ...&gt; XDR.encode(value) {:ok, &quot;abcdef&quot; &lt;&gt; &lt;&lt;0, 0&gt;&gt;} Each type&#39;s binary representation is determined by its own rules, as defined in the XDR spec. In the case of Opaque, the binary contents are passed through, with padding added to achieve an even multiple of 4 bytes. Variable-length types will be preceded by a four-byte integer describing the length of the contained value. iex&gt; {:ok, value} = XDR.build_value(XDR.build_type(XDR.Type.VariableOpaque), &quot;abcdef&quot;) ...&gt; XDR.encode(value) {:ok, &lt;&lt;0, 0, 0, 6&gt;&gt; &lt;&gt; &quot;abcdef&quot; &lt;&gt; &lt;&lt;0, 0&gt;&gt;} Note that type info is not contained in the binary representation, and is therefore required to decode the binary."},{"ref":"XDR.html#encode!/1","title":"XDR.encode!/1","type":"function","doc":"Just like XDR.encode/1, but returns raw binaries on success instead of tuples, and raises on failure."},{"ref":"XDR.html#extract_value/1","title":"XDR.extract_value/1","type":"function","doc":"XDR data structures created from XDR.build_value/2 and XDR.decode/2 include lots of type metadata, and the different types don&#39;t always store their inner state in the same way. XDR.extract_value/1 acts as a uniform way to pull out the underlying values as native elixir types. iex&gt; us_address = XDR.build_type(XDR.Type.Struct, ...&gt; street: XDR.build_type(XDR.Type.VariableOpaque), ...&gt; city: XDR.build_type(XDR.Type.VariableOpaque), ...&gt; state: XDR.build_type(XDR.Type.Opaque, 2), ...&gt; zip: XDR.build_type(XDR.Type.Opaque, 5) ...&gt; ) ...&gt; user_type = XDR.build_type(XDR.Type.Struct, ...&gt; name: XDR.build_type(XDR.Type.VariableOpaque), ...&gt; email: XDR.build_type(XDR.Type.VariableOpaque), ...&gt; address: us_address ...&gt; ) ...&gt; {:ok, user} = XDR.build_value(user_type, ...&gt; name: &quot;Marvin&quot;, ...&gt; email: &quot;marvin@megadodo.co&quot;, ...&gt; address: [ ...&gt; street: &quot;123 Shakedown St&quot;, ...&gt; city: &quot;New Orleans&quot;, ...&gt; state: &quot;LA&quot;, ...&gt; zip: &quot;70119&quot;, ...&gt; ] ...&gt; ) ...&gt; {:ok, user_info} = XDR.extract_value(user) ...&gt; user_info [ name: &quot;Marvin&quot;, email: &quot;marvin@megadodo.co&quot;, address: [ street: &quot;123 Shakedown St&quot;, city: &quot;New Orleans&quot;, state: &quot;LA&quot;, zip: &quot;70119&quot;, ] ]"},{"ref":"XDR.html#extract_value!/1","title":"XDR.extract_value!/1","type":"function","doc":"Just like XDR.extract_value/1, but returns raw values on success instead of tuples, and raises on failure."},{"ref":"XDR.html#t:ignored/0","title":"XDR.ignored/0","type":"type","doc":"Because the protocol function signatures must match for all types, arguments are sometimes unused. The ignored() type is used to annotate them."},{"ref":"XDR.Base.html","title":"XDR.Base","type":"module","doc":"Provides the ability to predefine and precompile specific XDR types for your application. Create a module in your app, and use XDR.Base. Your module will now have access to the define_type macro, as well as all of the functions on the main XDR module. See the README for an example."},{"ref":"XDR.Base.html#build_type/2","title":"XDR.Base.build_type/2","type":"function","doc":"Convenience function to build an XDR type. See XDR.build_type/2"},{"ref":"XDR.Base.html#comment/1","title":"XDR.Base.comment/1","type":"macro","doc":"A NOOP macro that allows for extensive documentation of defined types See the generated Stellar module"},{"ref":"XDR.Base.html#define_type/3","title":"XDR.Base.define_type/3","type":"macro","doc":"Define a named XDR type for your application by providing a name and type info. Once defined in your module, you can use type type name instead of a fully built XDR type in your module&#39;s functions such as build_value/2 and decode/1. The second and third arguments are the same as the first and second arguments of XDR.build_type/2."},{"ref":"XDR.Error.html","title":"XDR.Error","type":"exception","doc":"Errors explicitly created by XDR will usually be an XDR.Error. Also, errors triggered inside a complex data type will be wrapped and annotated with path info before being re-raised or returned to the user."},{"ref":"XDR.Error.html#wrap/1","title":"XDR.Error.wrap/1","type":"function","doc":""},{"ref":"XDR.Error.html#wrap/2","title":"XDR.Error.wrap/2","type":"function","doc":""},{"ref":"XDR.Padding.html","title":"XDR.Padding","type":"module","doc":"XDR encodings sometimes require padding. This module includes some helper functions to do those calculations."},{"ref":"XDR.Padding.html#padding/1","title":"XDR.Padding.padding/1","type":"function","doc":"The actual padding bytes required given the lenght of the content"},{"ref":"XDR.Padding.html#padding_length/1","title":"XDR.Padding.padding_length/1","type":"function","doc":"The number of bytes of padding required, given the length of the content"},{"ref":"XDR.Padding.html#t:padding/0","title":"XDR.Padding.padding/0","type":"type","doc":""},{"ref":"XDR.Padding.html#t:padding_len/0","title":"XDR.Padding.padding_len/0","type":"type","doc":""},{"ref":"XDR.Size.html","title":"XDR.Size","type":"module","doc":"Size utilities for various types"},{"ref":"XDR.Size.html#decode!/1","title":"XDR.Size.decode!/1","type":"function","doc":"Pull out the first 4 bytes of the input binary and decode it as an integer size. Return the size and remaining binary in a tuple iex&gt; XDR.Size.decode!(&lt;&lt;0, 0, 1, 1&gt;&gt; &lt;&gt; &quot;Hello&quot;) {0x00000101, &quot;Hello&quot;}"},{"ref":"XDR.Size.html#encode/1","title":"XDR.Size.encode/1","type":"function","doc":"Encode the size in a 32-byte binary iex&gt; XDR.Size.encode(0x00000101) &lt;&lt;0, 0, 1, 1&gt;&gt;"},{"ref":"XDR.Size.html#max/0","title":"XDR.Size.max/0","type":"function","doc":"The largest size available for sized types like array and opaque"},{"ref":"XDR.Size.html#valid?/1","title":"XDR.Size.valid?/1","type":"function","doc":""},{"ref":"XDR.Size.html#t:encoding/0","title":"XDR.Size.encoding/0","type":"type","doc":""},{"ref":"XDR.Size.html#t:t/0","title":"XDR.Size.t/0","type":"type","doc":"Non-negative, unsigned, 32-bit integers are used for sizes of various types, like opaque and array"},{"ref":"XDR.Type.html","title":"XDR.Type","type":"protocol","doc":""},{"ref":"XDR.Type.html#build_type/2","title":"XDR.Type.build_type/2","type":"function","doc":"Create a fully-configured type by supplying options such as length for an Array, fields for a Struct, or arms for a Union"},{"ref":"XDR.Type.html#build_value!/2","title":"XDR.Type.build_value!/2","type":"function","doc":"Given a type and a value compatible with the type, construct a new struct of the same type with the value applied"},{"ref":"XDR.Type.html#decode!/2","title":"XDR.Type.decode!/2","type":"function","doc":"Given a binary with the given type encoded at its head decode the XDR representation and return a tuple of the type with value and the remaining binary after the type"},{"ref":"XDR.Type.html#encode!/1","title":"XDR.Type.encode!/1","type":"function","doc":"Given a type with a value already applied, encode the value into its binary XDR representation"},{"ref":"XDR.Type.html#extract_value!/1","title":"XDR.Type.extract_value!/1","type":"function","doc":"Given a type with a value already applied, extract the value only."},{"ref":"XDR.Type.html#resolve_type!/2","title":"XDR.Type.resolve_type!/2","type":"function","doc":"Perform the necessary lookups using the custom_types map to resolve any subsidiary types. For example, a Struct&#39;s fields may be custom types (referenced via a string) that need to be resolved into the appropriate XDR.Type structs"},{"ref":"XDR.Type.html#t:t/0","title":"XDR.Type.t/0","type":"type","doc":"Any XDR.Type implementation"},{"ref":"XDR.Type.Array.html","title":"XDR.Type.Array","type":"module","doc":"A fixed-length array of some other type"},{"ref":"XDR.Type.Array.html#t:options/0","title":"XDR.Type.Array.options/0","type":"type","doc":""},{"ref":"XDR.Type.Array.html#t:t/0","title":"XDR.Type.Array.t/0","type":"type","doc":""},{"ref":"XDR.Type.Bool.html","title":"XDR.Type.Bool","type":"module","doc":"Boolean values are presented in Elixir as true / false and encoded as 1 / 0"},{"ref":"XDR.Type.Bool.html#t:encoding/0","title":"XDR.Type.Bool.encoding/0","type":"type","doc":""},{"ref":"XDR.Type.Bool.html#t:t/0","title":"XDR.Type.Bool.t/0","type":"type","doc":""},{"ref":"XDR.Type.Bool.html#t:value/0","title":"XDR.Type.Bool.value/0","type":"type","doc":""},{"ref":"XDR.Type.Const.html","title":"XDR.Type.Const","type":"module","doc":"Const: not really an XDR data type, but they can be declared the same way"},{"ref":"XDR.Type.Const.html#t:t/0","title":"XDR.Type.Const.t/0","type":"type","doc":""},{"ref":"XDR.Type.CustomType.html","title":"XDR.Type.CustomType","type":"module","doc":"A custom type is referenced by its name, as a bit string / binary"},{"ref":"XDR.Type.CustomType.html#register_type/4","title":"XDR.Type.CustomType.register_type/4","type":"function","doc":"Add a new type to the map, which is usually stored in a module that uses XDR.Base The map serves a a lookup from friendly type names to fully-defined XDR.Type.t() values"},{"ref":"XDR.Type.CustomType.html#t:t/0","title":"XDR.Type.CustomType.t/0","type":"type","doc":"A custom type is referenced by its name, as a bit string / binary"},{"ref":"XDR.Type.CustomType.html#t:type_map/0","title":"XDR.Type.CustomType.type_map/0","type":"type","doc":""},{"ref":"XDR.Type.Double.html","title":"XDR.Type.Double","type":"module","doc":"Double-precision (32 bit) floating-point number"},{"ref":"XDR.Type.Double.html#decode!/1","title":"XDR.Type.Double.decode!/1","type":"function","doc":"Decode the first 8 bytes of the given binary as a single-precision float and return the value along with the reamining binary as a tuple"},{"ref":"XDR.Type.Double.html#encode/1","title":"XDR.Type.Double.encode/1","type":"function","doc":"Encode the given float or %XDR.Type.Float{} as an 8-byte binary"},{"ref":"XDR.Type.Double.html#t:encoding/0","title":"XDR.Type.Double.encoding/0","type":"type","doc":""},{"ref":"XDR.Type.Double.html#t:t/0","title":"XDR.Type.Double.t/0","type":"type","doc":""},{"ref":"XDR.Type.Double.html#t:value/0","title":"XDR.Type.Double.value/0","type":"type","doc":""},{"ref":"XDR.Type.Enum.html","title":"XDR.Type.Enum","type":"module","doc":"Enum"},{"ref":"XDR.Type.Enum.html#t:options/0","title":"XDR.Type.Enum.options/0","type":"type","doc":""},{"ref":"XDR.Type.Enum.html#t:t/0","title":"XDR.Type.Enum.t/0","type":"type","doc":""},{"ref":"XDR.Type.Enum.html#t:value/0","title":"XDR.Type.Enum.value/0","type":"type","doc":""},{"ref":"XDR.Type.Float.html","title":"XDR.Type.Float","type":"module","doc":"Single-precision (32 bit) floating-point number"},{"ref":"XDR.Type.Float.html#decode!/1","title":"XDR.Type.Float.decode!/1","type":"function","doc":"Decode the first 4 bytes of the given binary as a single-precision float and return the value along with the reamining binary as a tuple"},{"ref":"XDR.Type.Float.html#encode/1","title":"XDR.Type.Float.encode/1","type":"function","doc":"Encode the given float or %XDR.Type.Float{} as an 4-byte binary"},{"ref":"XDR.Type.Float.html#t:encoding/0","title":"XDR.Type.Float.encoding/0","type":"type","doc":""},{"ref":"XDR.Type.Float.html#t:t/0","title":"XDR.Type.Float.t/0","type":"type","doc":""},{"ref":"XDR.Type.Float.html#t:value/0","title":"XDR.Type.Float.value/0","type":"type","doc":""},{"ref":"XDR.Type.HyperInt.html","title":"XDR.Type.HyperInt","type":"module","doc":"Signed 64-bit integer"},{"ref":"XDR.Type.HyperInt.html#decode!/1","title":"XDR.Type.HyperInt.decode!/1","type":"function","doc":"Decode the first 8 bytes of the binary as a hyperint and return that value along with the rest of the binary in a tuple"},{"ref":"XDR.Type.HyperInt.html#encode/1","title":"XDR.Type.HyperInt.encode/1","type":"function","doc":"Encode the given integer or XDR.Type.HyperInt struct as an 8-byte binary"},{"ref":"XDR.Type.HyperInt.html#t:encoding/0","title":"XDR.Type.HyperInt.encoding/0","type":"type","doc":""},{"ref":"XDR.Type.HyperInt.html#t:t/0","title":"XDR.Type.HyperInt.t/0","type":"type","doc":""},{"ref":"XDR.Type.HyperInt.html#t:value/0","title":"XDR.Type.HyperInt.value/0","type":"type","doc":""},{"ref":"XDR.Type.Int.html","title":"XDR.Type.Int","type":"module","doc":"Signed 32-bit integer"},{"ref":"XDR.Type.Int.html#decode!/1","title":"XDR.Type.Int.decode!/1","type":"function","doc":"Pull a raw integer out of the given binary, returning the value and remaining binary in a tuple"},{"ref":"XDR.Type.Int.html#encode/1","title":"XDR.Type.Int.encode/1","type":"function","doc":"Encode an integer or %XDR.Type.Int{} struct"},{"ref":"XDR.Type.Int.html#t:encoding/0","title":"XDR.Type.Int.encoding/0","type":"type","doc":""},{"ref":"XDR.Type.Int.html#t:t/0","title":"XDR.Type.Int.t/0","type":"type","doc":""},{"ref":"XDR.Type.Int.html#t:value/0","title":"XDR.Type.Int.value/0","type":"type","doc":""},{"ref":"XDR.Type.Opaque.html","title":"XDR.Type.Opaque","type":"module","doc":"Fixed-length binary data"},{"ref":"XDR.Type.Opaque.html#t:encoding/0","title":"XDR.Type.Opaque.encoding/0","type":"type","doc":""},{"ref":"XDR.Type.Opaque.html#t:t/0","title":"XDR.Type.Opaque.t/0","type":"type","doc":""},{"ref":"XDR.Type.Optional.html","title":"XDR.Type.Optional","type":"module","doc":"Optional"},{"ref":"XDR.Type.Optional.html#t:t/0","title":"XDR.Type.Optional.t/0","type":"type","doc":""},{"ref":"XDR.Type.Optional.html#t:value/0","title":"XDR.Type.Optional.value/0","type":"type","doc":""},{"ref":"XDR.Type.String.html","title":"XDR.Type.String","type":"module","doc":"Variable-length ASCII data This is almost the same as VariableOpaque, except the requirement that bytes be in the ASCII range"},{"ref":"XDR.Type.String.html#t:encoding/0","title":"XDR.Type.String.encoding/0","type":"type","doc":""},{"ref":"XDR.Type.String.html#t:t/0","title":"XDR.Type.String.t/0","type":"type","doc":""},{"ref":"XDR.Type.Struct.html","title":"XDR.Type.Struct","type":"module","doc":"Struct with each atom key pointing to an XDR type of its own"},{"ref":"XDR.Type.Struct.html#t:fields/0","title":"XDR.Type.Struct.fields/0","type":"type","doc":""},{"ref":"XDR.Type.Struct.html#t:t/0","title":"XDR.Type.Struct.t/0","type":"type","doc":""},{"ref":"XDR.Type.Union.html","title":"XDR.Type.Union","type":"module","doc":"A Union is a polymorphic type. It has a switch of type enum or int, whose value determines the type of the union&#39;s data. TODO finish support for a default arm"},{"ref":"XDR.Type.Union.html#get_value_type/2","title":"XDR.Type.Union.get_value_type/2","type":"function","doc":""},{"ref":"XDR.Type.Union.html#validate_type_options!/1","title":"XDR.Type.Union.validate_type_options!/1","type":"function","doc":""},{"ref":"XDR.Type.Union.html#t:options/0","title":"XDR.Type.Union.options/0","type":"type","doc":""},{"ref":"XDR.Type.Union.html#t:switch/0","title":"XDR.Type.Union.switch/0","type":"type","doc":""},{"ref":"XDR.Type.Union.html#t:switch_value/0","title":"XDR.Type.Union.switch_value/0","type":"type","doc":""},{"ref":"XDR.Type.Union.html#t:t/0","title":"XDR.Type.Union.t/0","type":"type","doc":""},{"ref":"XDR.Type.Union.html#t:value/0","title":"XDR.Type.Union.value/0","type":"type","doc":""},{"ref":"XDR.Type.UnsignedHyperInt.html","title":"XDR.Type.UnsignedHyperInt","type":"module","doc":"Unsigned 64-bit integer"},{"ref":"XDR.Type.UnsignedHyperInt.html#decode!/1","title":"XDR.Type.UnsignedHyperInt.decode!/1","type":"function","doc":"Decode the first 8 bytes of the binary as an unsigned int and return the value along with the remaining binary in a tuple"},{"ref":"XDR.Type.UnsignedHyperInt.html#encode/1","title":"XDR.Type.UnsignedHyperInt.encode/1","type":"function","doc":"Encode the given unsigned integer or struct as an 8-byte binary"},{"ref":"XDR.Type.UnsignedHyperInt.html#t:encoding/0","title":"XDR.Type.UnsignedHyperInt.encoding/0","type":"type","doc":""},{"ref":"XDR.Type.UnsignedHyperInt.html#t:t/0","title":"XDR.Type.UnsignedHyperInt.t/0","type":"type","doc":""},{"ref":"XDR.Type.UnsignedHyperInt.html#t:value/0","title":"XDR.Type.UnsignedHyperInt.value/0","type":"type","doc":""},{"ref":"XDR.Type.UnsignedInt.html","title":"XDR.Type.UnsignedInt","type":"module","doc":"Unsigned 32-bit integer"},{"ref":"XDR.Type.UnsignedInt.html#decode!/1","title":"XDR.Type.UnsignedInt.decode!/1","type":"function","doc":"Decode the first 4 bytes of the binary into an unsigned int and return the value along with the remaining binary in a tuple"},{"ref":"XDR.Type.UnsignedInt.html#encode/1","title":"XDR.Type.UnsignedInt.encode/1","type":"function","doc":"Encode the integer or XDR.Type.UnsignedInt struct as a 4-byte binary"},{"ref":"XDR.Type.UnsignedInt.html#t:encoding/0","title":"XDR.Type.UnsignedInt.encoding/0","type":"type","doc":""},{"ref":"XDR.Type.UnsignedInt.html#t:t/0","title":"XDR.Type.UnsignedInt.t/0","type":"type","doc":""},{"ref":"XDR.Type.UnsignedInt.html#t:value/0","title":"XDR.Type.UnsignedInt.value/0","type":"type","doc":""},{"ref":"XDR.Type.VariableArray.html","title":"XDR.Type.VariableArray","type":"module","doc":"A variable-length array of some other type"},{"ref":"XDR.Type.VariableArray.html#t:options/0","title":"XDR.Type.VariableArray.options/0","type":"type","doc":""},{"ref":"XDR.Type.VariableArray.html#t:t/0","title":"XDR.Type.VariableArray.t/0","type":"type","doc":""},{"ref":"XDR.Type.VariableOpaque.html","title":"XDR.Type.VariableOpaque","type":"module","doc":"Variable-length binary data"},{"ref":"XDR.Type.VariableOpaque.html#t:encoding/0","title":"XDR.Type.VariableOpaque.encoding/0","type":"type","doc":""},{"ref":"XDR.Type.VariableOpaque.html#t:t/0","title":"XDR.Type.VariableOpaque.t/0","type":"type","doc":""},{"ref":"XDR.Type.Void.html","title":"XDR.Type.Void","type":"module","doc":"Void"},{"ref":"XDR.Type.Void.html#t:encoding/0","title":"XDR.Type.Void.encoding/0","type":"type","doc":""},{"ref":"XDR.Type.Void.html#t:t/0","title":"XDR.Type.Void.t/0","type":"type","doc":""},{"ref":"readme.html","title":"XDR","type":"extras","doc":"Package Support CI (master) Coverage XDR XDR is an open data format for serializing and de-serializing structured data based on shared definitions, defined in RFC 4506. This library aims to provide an idiomatic interface for working with XDR data in Elixir."},{"ref":"readme.html#documentation","title":"XDR - Documentation","type":"extras","doc":"Detailed documentation and examples can be found at https://hexdocs.pm/exdr."},{"ref":"readme.html#installation","title":"XDR - Installation","type":"extras","doc":"Add exdr to your list of dependencies in mix.exs: def deps do [ {:exdr, &quot;~&gt; 0.1.1&quot;} ] end"},{"ref":"readme.html#basic-usage","title":"XDR - Basic Usage","type":"extras","doc":"To work with XDR, an application needs to be able to do five things: Define the types and structure of the data Create instances of the data types with specific values Encode that structured data into a binary representation Decode a binary representation into structured, typed data Extract the raw values from the data structures The main XDR module provides functions for each of these operations. These examples illustrate their usage: Building data types Fully-determined types can be built by configuring and combining the predefined XDR types. int_type = XDR.build_type(XDR.Type.Int) name_type = XDR.build_type(XDR.Type.VariableOpaque) five_ints_type = XDR.build_type(XDR.Type.Array, type: int_type, length: 5) student_type = XDR.build_type(XDR.Type.Struct, name: name_type, quiz_scores: five_ints_type, homework_scores: five_ints_type ) The configuration required by each built-in type varies. See the documentation for more details. Adding specific values Once the data types are defined, values can be built. Again, the applicable values depend on the data types. {:ok, single_score} = XDR.build_value(int_type, 92) {:ok, single_name} = XDR.build_value(name_type, &quot;Student A&quot;) Complex data types can be built up all at once, rather than having to initialize each subsidiary value: {:ok, student_a} = XDR.build_value(student_type, name: &quot;Student A&quot;, quiz_scores: [100, 93, 60, 88, 100], homework_scores: [66, 80, 100, 99, 0] ) Encoding data Once data structures have been defined and specific values created, they can be encoded into a binary XDR representation: {:ok, single_score_encoding} = XDR.encode(single_score) {:ok, single_name_encoding} = XDR.encode(single_name) {:ok, student_a_encoding} = XDR.encode(student_a) Decoding data Any XDR implementation with access to the same type definitions will be able to decode the binary representations into structured data. {:ok, single_score_decoded} = XDR.decode(int_type, single_score_encoding) {:ok, single_name_decoded} = XDR.decode(name_type, single_name_encoding) {:ok, student_a_decoded} = XDR.decode(student_type, student_a_encoding) # Upon decoding, these values are fully-fledged XDR types %XDR.Type.Struct{fields: fields} = student_a_decoded Extracting underlying values To use the data, we probably want to extract the raw values from the XDR metadata: {:ok, student_a_data} = XDR.extract_value(student_a_decoded) # this should match what we put in originally [ name: &quot;Student A&quot;, quiz_scores: [100, 93, 60, 88, 100], homework_scores: [66, 80, 100, 99, 0] ] = student_a_data"},{"ref":"readme.html#custom-xdr-type-definitions","title":"XDR - Custom XDR Type Definitions","type":"extras","doc":"When building complex apps, it&#39;s not convenient to have to build up the types manually every time time they&#39;re needed, so an application can predefine its XDR types and compile them into the application. The XDR.Base module provides the define_type macro for registering and accessing application-specific types. defmodule MyXDR @moduledoc &quot;&quot;&quot; We can define custom types in our own module and then work with them through that module. All of the functions demonstrated above on XDR are available on MyXDR, and we can use simple string names to reference our predefined XDR types. &quot;&quot;&quot; use XDR.Base # `define_type` works just like `build_type` with an extra `name` parameter at the front define_type(&quot;Student&quot;, Struct, name: &quot;Name&quot;, quiz_scores: &quot;Scores&quot;, homework_scores: &quot;Scores&quot;, ) define_type(&quot;Scores&quot;, Array, type: &quot;Score&quot;, length: 5 ) define_type(&quot;Score&quot;, Int) define_type(&quot;Name&quot;, VariableOpaque) end # using the predefined types in our code {:ok, student_b} = MyXDR.build_value!(&quot;Student&quot;, name: &quot;Student B&quot;, quiz_scores: [93, 60, 88, 100, 84], homework_scores: [80, 100, 99, 0, 90] ) {:ok, encoded_student_b} = MyXDR.encode(student_b) {:ok, decoded_student_b} = MyXDR.decode(&quot;Student&quot;, encoded_student_b) For a real-life example, see the Stellar.XDR module, which was generated from the Stellar type definitions using the xdrgen tool."},{"ref":"readme.html#contributing-and-development","title":"XDR - Contributing and Development","type":"extras","doc":"See CONTRIBUTING.md for guidance on how to develop for this library. Bug reports and pull requests are welcome on GitHub at https://github.com/revelrylabs/exdr. Check out CONTRIBUTING.md for more info. Everyone is welcome to participate in the project. We expect contributors to adhere to the Contributor Covenant Code of Conduct (see CODE_OF_CONDUCT.md)."},{"ref":"readme.html#releases","title":"XDR - Releases","type":"extras","doc":"See RELEASES.md for details about the release process."},{"ref":"readme.html#license","title":"XDR - License","type":"extras","doc":"See LICENSE for details."}]